<?php 
/**
 * @package ProRadio Podcast Importer
 * 
 * Admin Settings:
 * Create forms for the admin
 * 
 * */


if ( ! defined( 'ABSPATH' ) ) {
	die( '-1' );
}


if(!function_exists('proradio_pimp_form_generator')){
	function proradio_pimp_form_generator($title, $controls,  $nonce_action , $nonce_name){
		$user = wp_get_current_user();
		$allowed_roles = array( 'administrator' );
		if( !array_intersect($allowed_roles, $user->roles ) ) {
			echo "This user is not allowed to access this page";
			return;
		}





		/**
		 * =====================================================
		 * 
		 * SAVE Admin settings
		 * 
		 * =====================================================
		 **/
		$msg = false;
		$service = function_exists('prpimp_whmcs_check_service_standalone') ? prpimp_whmcs_check_service_standalone(base64_decode(PRORADIO_PIMP_SERVICE_NAME))     : false;
		if( $service ){ $service = strtotime($service) > time(); }
		if ( ! empty( $_POST ) ) {
			if(isset($_POST['proradio_pimp-submitform'])){
				if(!check_admin_referer( $nonce_action, $nonce_name )){
					echo 'Invalid request';
				} else {
					if( ! $service ) {
						foreach( $controls as $control ){
							if(array_key_exists('default', $control) && array_key_exists('advanced', $control) && $control['advanced'] == 1 ){
								update_option($control['id'], $control['default']);
							}
						}
					}
					foreach( $controls as $control ){
						if(array_key_exists('id',$control)){
							$varname = $control['id'];
						}
						if(array_key_exists('advanced', $control)){
							if( ! $service && $control['advanced'] == 1 ){
								$control['type'] = 'pre';
								
							}
						}
						switch ($control['type']){
							case 'licensekey':

								if (strpos( $_POST[ $varname ] , prpimp_iid() ) !== false) {
									$licensekey = esc_attr(trim($_POST[ $varname ]));
									$localkey 	= false;
									if( $licensekey == get_option( $varname) ){
										$localkey 	= get_option( 'prpimp_ack_' . prpimp_iid() );
									}
									$results = prpimp_custom_whmcs_check_license( $licensekey, $localkey);
									$valid = false;
									if( $results && $results['status'] == 'Active'){
										$site = parse_url(get_site_url());
										if( in_array( $site['host'] , explode(',', $results['validdomain'])) ){
											if( strtotime($results['nextduedate']) >  time() ){
												$valid = true;
											}
										}
									}
									if( $valid ){
										update_option( $varname, $licensekey );	
									} else {
										update_option( $varname, '' );	
									}
									
								} else {
									update_option( $varname, '' );	
								}
								break;
							case 'number':
								if(isset($varname) && isset($_POST[$varname])){
									$num = wp_kses_post( wp_unslash( $_POST[$varname]) ) ;
									$num = intval($num);
									if(isset( $control['max'] ) && $num > $control['max']){
										$num = $control['max'];
									}
									if(isset( $control['min'] ) && $num < $control['min']){
										$num = $control['min'];
									}
									update_option($varname, $num );
								}
								break;
							case 'pre':
								// Disregards
								break;
							case 'checkbox':
								if(isset($_POST[$varname])){
									if($_POST[$varname] == 'on'){
										update_option($varname, 1);
										
									} 
								} else {
									update_option($varname, 0 );
								}
								
								
								break;
							default:
								if(isset($varname) && isset($_POST[$varname])){
									update_option($varname, wp_kses(wp_unslash($_POST[$varname] ) , array()));
								}
						}

						
						if(array_key_exists('advanced',$control) ){
							if( $control['advanced'] == 1 && !$service){
								if(array_key_exists('default',$control)){
									update_option($varname, $control['default']);
								} else {
									delete_option($varname);
								}
							}
						}
					}
				}
			}
		}


		/**
		 * =====================================================
		 * 
		 * OUTPUT
		 * 
		 * =====================================================
		 **/

		?>
		
			
			<h1 class="proradio_pimp_settings-modaltitle"><?php echo esc_html( $title ); ?></h1>

			<?php 
			 if( $msg ){
			 	?>
			 	<div id="setting-success-tgmpa" class="notice notice-success " style="margin-left:0;padding: 30px;"><?php 
			 	 echo wp_kses_post($msg);
			 	?></div>
			 	<?php 
			 }
			?>
			<div class="proradio_pimp__row">
			<form method="post" class="proradio_pimp__form" action="">

				<div class="proradio_pimp-table-container">
					<table class="form-table" role="presentation">
						<tbody>
							<?php 
							/**
							 * =====================================================
							 * 
							 * Fields loop
							 * 
							 * =====================================================
							 **/
							
							
		
							foreach( $controls as $control ){
								if(array_key_exists('advanced', $control)){
									if( ! $service && $control['advanced'] == 1 ){
										$control['type'] = 'premium';
									}
								}
								switch ($control['type']){
									/**
									 * ====================================
									 * Premium
									 * ====================================
									 **/
									case 'premium':
										?>
										<tr class="proradio_pimp__row">
											<th><?php echo esc_html( $control['label'] ); ?></th>
											<td>
												<?php echo ( array_key_exists('description', $control) ) ? '<p>'.wp_kses_post( $control['description'] ).'</p>' : ''; ?>
												<p><strong><a href="<?php echo esc_url( prpimp_promo_url()); ?>" target="_blank"><?php echo esc_html( prpimp_promo_text() ); ?></a></strong></p>
											</td>
										</tr>
										<?php
										break;
									/**
									 * ====================================
									 * Section
									 * ====================================
									 **/
									case 'section':
										?>
										<tr class="proradio_pimp__row">
											<th colspan="2"><h2><?php echo esc_attr( $control['label']); ?></h2></th>
										</tr>
										<?php 
										break;

									/**
									 * ====================================
									 * Product select
									 * ====================================
									 **/

									case 'productselect':
									
										?>
										<tr class="proradio_pimp__row">
											<th><?php echo esc_attr( $control['label']); ?></th>
											<td>

												<select name="<?php echo esc_attr($control['id']); ?>" id="<?php echo esc_attr($control['id']); ?>">
												<?php
												$args = array(
													'post_type' => "product",
													'posts_per_page' => -1,
													'paged' => 1,
												);
												wp_reset_postdata();
												$wp_query = new WP_Query( $args );
												if ( $wp_query->have_posts() ) : while ( $wp_query->have_posts() ) : $wp_query->the_post();
													$post = $wp_query->post;
													setup_postdata( $post );
													$product = wc_get_product( $post->ID );
													?>
													<option value="<?php echo esc_attr($post->ID); ?>" <?php if( get_option(  $control['id'] ) == $post->ID){ echo 'selected';} ?>><?php echo esc_html($post->post_title); ?></option>
													<?php
												endwhile;
												else:
													?>
													<option value=""><?php echo esc_html($post->post_title); ?></option>
													<?php
												endif;
												wp_reset_postdata();

												 ?>
												</select>
											 <?php echo ( array_key_exists('description', $control) ) ? '<p>'.wp_kses_post( $control['description'] ).'</p>' : ''; ?>
											</td>
										</tr>
										<?php 
										break;



									/**
									 * ====================================
									 * Taxonomy Select
									 * ====================================
									 **/
									case 'taxonomyselect':
									    $taxonomies = get_taxonomies(array('public' => true), 'objects'); // Get public taxonomies
									    $saved_value = get_option($control['id'], false); // Get the saved option (false if not set)
									    $default_value = isset($control['default']) ? $control['default'] : ''; // Get default if exists

									    // Determine the selected value: use saved value if it exists, otherwise fallback to default
									    $selected_value = ($saved_value !== false) ? $saved_value : $default_value;
									    ?>
									    <tr class="proradio_pimp__row">
									        <th><?php echo esc_attr($control['label']); ?></th>
									        <td>
									            <select name="<?php echo esc_attr($control['id']); ?>" id="<?php echo esc_attr($control['id']); ?>">
									                <option value="">-</option>
									                <?php foreach ($taxonomies as $taxonomy) : ?>
									                    <option value="<?php echo esc_attr($taxonomy->name); ?>" 
									                        <?php selected($selected_value, $taxonomy->name); ?>>
									                        <?php echo esc_html($taxonomy->label); ?>
									                    </option>
									                <?php endforeach; ?>
									            </select>
									            <?php echo !empty($control['description']) ? '<p>' . wp_kses_post($control['description']) . '</p>' : ''; ?>
									        </td>
									    </tr>
									    <?php
									    break;


									/**
									 * ====================================
									 * Post Type Select
									 * ====================================
									 **/
									case 'posttypeselect':
									    $post_types = get_post_types(array('public' => true), 'objects'); // Get public post types
									    $saved_value = get_option($control['id'], false); // Get the saved option (false if not set)
									    $default_value = isset($control['default']) ? $control['default'] : ''; // Get default if exists

									    // Determine the selected value: use saved value if it exists, otherwise fallback to default
									    $selected_value = ($saved_value !== false) ? $saved_value : $default_value;
									    ?>
									    <tr class="proradio_pimp__row">
									        <th><?php echo esc_attr($control['label']); ?></th>
									        <td>
									            <select name="<?php echo esc_attr($control['id']); ?>" id="<?php echo esc_attr($control['id']); ?>">
									                <option value="">-</option>
									                <?php foreach ($post_types as $post_type) : ?>
									                    <option value="<?php echo esc_attr($post_type->name); ?>" 
									                        <?php selected($selected_value, $post_type->name); ?>>
									                        <?php echo esc_html($post_type->label); ?>
									                    </option>
									                <?php endforeach; ?>
									            </select>
									            <?php echo !empty($control['description']) ? '<p>' . wp_kses_post($control['description']) . '</p>' : ''; ?>
									        </td>
									    </tr>
									    <?php
									    break;

									/**
									 * ====================================
									 * Page select
									 * ====================================
									 **/

									case 'pageselect':
										$posttype = array_key_exists('posttype',  $control ) ?  $control['posttype'] : 'page';
										$args = array(
											'post_type' => $posttype,
											'posts_per_page' => -1,
											'paged' => 1,
										);
										wp_reset_postdata();
										$wp_query = new WP_Query( $args );
								
										?>
										<tr class="proradio_pimp__row">
											<th><?php echo esc_attr( $control['label']); ?></th>
											<td>
												<?php  
												if ( $wp_query->have_posts() ){
												?>

													<select name="<?php echo esc_attr($control['id']); ?>" id="<?php echo esc_attr($control['id']); ?>">
														<option value="">-</option>
													<?php
													if ( $wp_query->have_posts() ) : while ( $wp_query->have_posts() ) : $wp_query->the_post();
														$post = $wp_query->post;
														setup_postdata( $post );
														?>
														<option value="<?php echo esc_attr($post->ID); ?>" <?php if( get_option(  $control['id'] ) == $post->ID){ echo 'selected';} ?>><?php echo esc_html($post->post_title); ?></option>
														<?php
													endwhile;
													else:
														?>
														<option value=""><?php echo esc_html($post->post_title); ?></option>
														<?php
													endif;
													wp_reset_postdata();

													 ?>
													</select>
												<?php  } else {
													echo 'There are no radio shows yet';
												}

												?>
											 <?php echo ( array_key_exists('description', $control) ) ? '<p>'.wp_kses_post( $control['description'] ).'</p>' : ''; ?>
											</td>
										</tr>
										<?php 
										break;

									/**
									 * ====================================
									 * Checkbox
									 * ====================================
									 **/

									case 'checkbox':
										?>
										<tr class="proradio_pimp__row">
											<th><?php echo esc_attr( $control['label']); ?></th>
											<td>
												<input id="<?php echo esc_attr( $control['id'] ); ?>" name="<?php echo esc_attr(  $control['id'] ); ?>"  type="checkbox" <?php if (  array_key_exists('default', $control)      &&      get_option(  $control['id'], $control['default'])){ ?> checked <?php } ?>>
												<?php
												echo ( array_key_exists('description', $control) ) ? wp_kses_post( $control['description'] ) : ''; 

												if( $control['id'] === 'proradio_pimp-enable-songhistory' ){
													if(  wp_next_scheduled ( 'proradio_pimp_event_songhistory' ) ){
														echo '| <strong>';esc_html_e( 'Service enabled. Last run: ', 'proradio-podcast-importer' ); 
														echo wp_kses_post( get_option( 'proradio_pimp_cron_log' )); 
														echo ' </strong> Enabled channels:';
														

															$wp_query = new WP_Query( array(
																'post_type' => 'radiochannel',
																'meta_key' => 'proradio_pimp-store-history',
																'meta_value' => '1',
																'limit' => 5,
																'order' => 'ASC'
															));
															if ( $wp_query->have_posts() ) : while ( $wp_query->have_posts() ) : $wp_query->the_post();
																$post = $wp_query->post;
																?>
																| <a href="<?php echo esc_url( get_edit_post_link( $post->ID )); ?>" target="_blank"><?php the_title(); ?></a>
																<?php 
																endwhile; else:
																esc_html_e( 'No radio is enabled for history. Edit radio channel to enable history', 'proradio-podcast-importer' );
																
															endif;
															wp_reset_postdata();
													}
												}

											

												?>
											</td>
										</tr>
										<?php 
										break;

									/**
									 * ====================================
									 * Radio
									 * ====================================
									 * 
									 **/

									case 'radio':
										?>
										<tr class="proradio_pimp__row">
											<th><?php echo esc_attr( $control['label']); ?></th>
											<td>
												<?php
													echo ( array_key_exists('description', $control) ) ? '<p>'.wp_kses_post( $control['description'] ).'</p>' : ''; 
													?>
												<?php 
												// echo $var.' : STORED OPTION: '.get_option(  $control['id'] );

												foreach($control['options'] as $var => $val){

													?>
													<p>
														<input  name="<?php echo esc_attr(  $control['id'] ); ?>"  type="radio" id="<?php echo esc_attr( $var ); ?>" value="<?php echo esc_attr( $var ); ?>" <?php if ( $var == get_option(  $control['id'], $control['default'] ) ){ ?> checked <?php } ?>>
														<label for="<?php echo esc_attr( $var ); ?>"><?php echo esc_attr( $val ); ?></label>
													</p>
													
													<?php										
												}
												?>

											</td>
										</tr>
										<?php 
										break;

									/**
									 * 
									 * ====================================
									 * Textarea
									 * ====================================
									 * 
									 **/

									case 'textarea': 
										?>
										<tr class="proradio_pimp__row">
											<th><?php echo esc_html( $control['label'] ); ?></th>
											<td>
												<textarea id="<?php echo esc_attr($control['id']); ?>" name="<?php echo esc_attr($control['id']); ?>"  type="textarea"><?php echo wp_kses_post( wp_unslash( get_option( $control['id'], $control['default']) ) ); ?></textarea>
												<?php
												echo ( array_key_exists('description', $control) ) ? '<p>'.wp_kses_post( $control['description'] ).'</p>' : ''; 
												?>
											</td>
										</tr>
										<?php
										break;

									/**
									 * 
									 * ====================================
									 * Textarea
									 * ====================================
									 * 
									 **/

									case 'code': 
										?>
										<tr class="proradio_pimp__row">
											<th><?php echo esc_html( $control['label'] ); ?></th>
											<td>
												<textarea id="<?php echo esc_attr($control['id']); ?>" name="<?php echo esc_attr($control['id']); ?>"  type="textarea" class="proradio-pimp-code"><?php echo wp_kses_post( wp_unslash( get_option( $control['id'], $control['default']) ) ); ?></textarea>
												<?php
												echo ( array_key_exists('description', $control) ) ? '<p>'.wp_kses_post( $control['description'] ).'</p>' : ''; 
												?>
											</td>
										</tr>
										<?php
										break;
									/**
									 * ====================================
									 * Text
									 * ====================================
									 **/
									case 'text':
										?>
										<tr class="proradio_pimp__row">
											<th><?php echo esc_html( $control['label'] ); ?></th>
											<td><input id="<?php echo esc_attr($control['id']); ?>" name="<?php echo esc_attr($control['id']); ?>"  type="text" value="<?php echo esc_attr( get_option( $control['id'], $control['default']) ); ?>"><?php   ?>
												<?php
												echo ( array_key_exists('description', $control) ) ? '<p>'.wp_kses_post( $control['description'] ).'</p>' : ''; 
												?>
											</td>
										</tr>
										<?php
										break;


									/**
									 * ====================================
									 * Licensekey
									 * ====================================
									 **/
									case 'licensekey':



										?>
										<tr class="proradio_pimp__row">
											<th><?php echo esc_html( $control['label'] ); ?></th>
											<td><input id="<?php echo esc_attr($control['id']); ?>" name="<?php echo esc_attr($control['id']); ?>"  type="text" value="<?php echo esc_attr( get_option( $control['id'] ) ); ?>"><?php   ?>
												<?php
												echo ( array_key_exists('description', $control) ) ? '<p>'.wp_kses_post( $control['description'] ).'</p>' : ''; 
												?>
											</td>
										</tr>
										<?php
										break;

									/**
									 * ====================================
									 * Text
									 * ====================================
									 **/
									case 'color':
										?>
										<tr class="proradio_pimp__row">
											<th><?php echo esc_html( $control['label'] ); ?></th>
											<td><input id="<?php echo esc_attr($control['id']); ?>" name="<?php echo esc_attr($control['id']); ?>"  type="color" value="<?php echo esc_attr( get_option( $control['id'], $control['default']) ); ?>"><?php   ?>
												<?php
												echo ( array_key_exists('description', $control) ) ? '<p>'.wp_kses_post( $control['description'] ).'</p>' : ''; 
												?>
											</td>
										</tr>
										<?php
										break;

									/**
									 * ====================================
									 * Number
									 * ====================================
									 **/
									case 'number':
										?>
										<tr class="proradio_pimp__row">
											<th><?php echo esc_html( $control['label'] ); ?></th>
											<td><input id="<?php echo esc_attr($control['id']); ?>" name="<?php echo esc_attr($control['id']); ?>"  type="number" value="<?php echo esc_attr( get_option( $control['id'], $control['default']) ); ?>" >
												<?php
												echo ( array_key_exists('description', $control) ) ? '<p>'.wp_kses_post( $control['description'] ).'</p>' : ''; 
												?>
											</td>
										</tr>
										<?php
										break;

									/**
									 * 
									 * ====================================
									 * None or wrong
									 * ====================================
									 * 
									 **/
									default: 
										echo esc_html( $control['label'] );
								}
							}
						?>
						<tr>
						<td colspan="2">
							<?php wp_nonce_field( $nonce_action, $nonce_name, true, true ); ?>
							<input type="submit" name="submit" value="Save"  class="button button-primary" />
						</td>
						</tr>
						</tbody>
					</table>
				</div>
				<input type="hidden" name="proradio_pimp-submitform" value="1">
			</form>
		</div>
		<?php 
	}
}


if(!function_exists('proradio_pimp_add_cron_interval')){
add_filter( 'cron_schedules', 'proradio_pimp_add_cron_interval' );
function proradio_pimp_add_cron_interval( $schedules ) {
	$interval = 120;
	$min = get_option( 'proradio_pimp-cron-time', '3' );
	if($min ){
		switch ( $min ){
			case "1":
				$interval = 60;
				break;
			case "2":
				$interval = 120;
				break;
			case "3":
				$interval = 180;
				break;
			case "5":
			default:
				$interval = 300;
				break;
		}
	}
    $schedules['every-sidekick-custom'] = array(
            'interval'  => $interval, // time in seconds
            'display'   => 'Pro Radio Sidekick Interval'
    );
    return $schedules;
}}