// nothing yet

/**
 * =======================================
 * Admin script for proradio_pimp admin
 * =======================================
 * */
(function($) {
	"use strict";

	$.Prpimp = {
		allowProcess: true, // allow processing queue ( not stopped )
		totalImport: 0,
		totalSkipped: 0,
		totalAnalyzed: 0,
		totalRequested: 0,
		out: function(text){
			var $logContainer = $('.proradio_pimp-ajax-importer-output');
			$logContainer.append('<p>' + text + '</p>');
			$logContainer.scrollTop($logContainer.prop("scrollHeight"));
		},
		processTaskQueue: function(taskList, totalItems, index = 0) {
			if( true === $.Prpimp.allowProcess ){
				console.log( "processing task "+index);
				// Stop if no more tasks
				if (index >= taskList.length) {
					$('[data-prpimp-ajaximport]').removeClass('disabled').css({'pointer-events':'initial'});
					$.Prpimp.updateWorkingState(true);
					$.Prpimp.out('✅  Done! Imported: ' + $.Prpimp.totalImport +' episodes. Skipped: '+ $.Prpimp.totalSkipped + ' Analyzed:'+ $.Prpimp.totalAnalyzed +' Requested:'+ $.Prpimp.totalRequested );
					$.Prpimp.progress(100);
					$('.data-prpimp-ajaximport-working').removeClass('active');
					$('[data-prpimp-ajaxstop]').hide();
					return;
				}

				var task_id = taskList[index]; // Get current task
				var currentIndex = index + 1; // For progress calculation

				$.Prpimp.generateTasksQueue(task_id, totalItems, currentIndex, function() {
					var progress = (currentIndex / totalItems) * 100;
					$.Prpimp.progress(progress);
					$.Prpimp.processTaskQueue(taskList, totalItems, currentIndex); // Process next task
				});
			}
		},


		updateWorkingState: function(isWorking) {
	        if (isWorking) {
	            $('.data-prpimp-ajaximport-working').addClass('active');
	        } else {
	            $('.data-prpimp-ajaximport-working').removeClass('active');
	        }
	    },

		// For each rss feed, call an ajax function that can download the xml, split ito tasks and delete the task once done
		generateTasksQueue: function(task_id, totalItems, index, callback) {
			 $.ajax({
				url: proradio_pimp_ajax_var.url, // WordPress global AJAX handler
				type: 'POST',
				dataType: 'json',
				data: {
					action: 'prpimp_generate_tasks_queue',
					task_id: task_id,
					nonce: proradio_pimp_ajax_var.nonce
				},
				beforeSend: function() {
					$.Prpimp.out('<p>Generating task queue for ' + task_id + '</p>');
				},
				success: function(jsonData) {
					var total_imported = jsonData.total_imported || 0;
					var skipped = jsonData.skipped || 0;
					var total_items = jsonData.total_items || 0;
					var limit = jsonData.limit || 0;
					var result = jsonData.result || 'fail';

					// Update global counters
					$.Prpimp.totalImport += total_imported;
					$.Prpimp.totalSkipped += skipped;
					$.Prpimp.totalAnalyzed += total_items;
					$.Prpimp.totalRequested += limit;

					if (result === 'success') {
						$.Prpimp.out('Task ' + task_id + ' completed: ' + total_imported + ' imported, ' + skipped + ' skipped.');
					} else {
						$.Prpimp.out('Task ' + task_id + ' failed.');
					}

					// Call the callback function to process the next task
					if (typeof callback === "function") {
						callback();
					}
				},
				error: function(xhr, status, error) {
					$.Prpimp.out('Error processing task ' + task_id + ': ' + error);
					if (typeof callback === "function") {
						callback(); // Continue to the next task even if this one fails
					}
				}
			});

		},
		progress: function(percent){
			if( $.Prpimp.allowProcess ){
				$('.proradio_pimp-progress-bar').addClass('active').css({'width': percent+'%'});
				if( percent == 100){
					$('.proradio_pimp-progress-bar').addClass('done');
				}
			}
		},
		// Link click to start
		buttonClick: function(){
		    $(document).on('click', '[data-prpimp-ajaximport]', function(e) {
		        e.preventDefault();
		        
		        var button = $(this);
		        button.addClass('disabled').css({'pointer-events':'none'});

		        $.Prpimp.progress(5);
		        $.Prpimp.updateWorkingState(true);
		         $('[data-prpimp-ajaxstop]').show();
		         $('.proradio_pimp-progress-container, .proradio_pimp-ajax-importer-output').addClass('active');

		        // Generate the list of feed sources
		        $.ajax({
		            url: proradio_pimp_ajax_var.url, 
		            type: 'POST',
		            dataType: 'json',
		            data: {
		                action: 'prpimp_get_tasks_list',
		                nonce: proradio_pimp_ajax_var.nonce
		            },
		            beforeSend: function() {
		                $.Prpimp.progress(10);
		                $.Prpimp.out('<p>Extracting list of active feed sources</p>');
		            },
		            success: function(jsonData) {
		            	console.log( jsonData );
		                var taskList = Object.keys(jsonData);
		                var totalItems = taskList.length;
		                $.Prpimp.out('Found ' + totalItems + ' feeds to import');

		                if (totalItems > 0) {
		                    $.Prpimp.processTaskQueue(taskList, totalItems);
		                } else {
		                    $.Prpimp.out('No tasks found.');
		                    button.removeClass('disabled').text('Done!').css({'pointer-events':'none'});
		                }
		            },
		            error: function(xhr, status, error) {
		                $.Prpimp.out('Error: ' + error);
		            }
		        });
		    });

		   $('[data-prpimp-ajaxstop]').on('click', function() {
		   		var btn = $(this);
		   		$.Prpimp.allowProcess = false;
		        $.ajax({
		            url: proradio_pimp_ajax_var.url,
		            type: 'POST',
		            dataType: 'json',
		            data: { 
		            	action: 'prpimp_stop_import', 
		            	nonce: proradio_pimp_ajax_var.nonce 
		        	},
		        	success: function(jsonData) {
		        		console.log( jsonData );
		        		$.Prpimp.out('Stop ' + jsonData );
		                $.Prpimp.updateWorkingState(false);
		                $('[data-prpimp-ajaximport]').removeClass('disabled').css({'pointer-events':'initial'});
		                btn.hide();
		            },
		            error: function(xhr, status, error) {
		                $.Prpimp.out('Error: ' + error);
		            }
		        });
		    });

		   $('#prpimp_delete_button').on('click', function(event) {
		        var confirmationText = $('input[name="proradio_pimp_confirm_delete"]').val();
		        
		        // Get the confirmation messages from the form's data attributes
		        var confirmMessage = $('.proradio_pimp__form__delete').data('confirm-message');
		        var trashConfirmation = $('.proradio_pimp__form__delete').data('trash-confirmation');
		        
		        // Check if 'DELETE' was typed in the confirmation field
		        if (confirmationText !== 'DELETE') {
		            alert(confirmMessage); // Show alert with the confirmation message
		            event.preventDefault(); // Prevent form submission
		        } else {
		            // Show the confirmation dialog
		            var confirmDelete = confirm(trashConfirmation); // Show confirm dialog
		            if (!confirmDelete) {
		                event.preventDefault(); // Prevent form submission if user cancels
		            }
		        }
		    });


		},
		init: function(){
			console.log('Podcast Importer Init');
			$.Prpimp.buttonClick();
		}
	};

	/**====================================================================
	 *
	 *	Page Ready Trigger
	 * 
	 ====================================================================*/
	jQuery(document).ready(function() {
		$.Prpimp.init();
	});
})(jQuery);