<?php 
/**
 * @package Pro.Radio Podcast Importer
 * Run all import tasks
 * 
 * */
if ( ! defined( 'ABSPATH' ) ) { exit; }


/***
 * =============================================
 * Extract tasks and feed URLs
 * =============================================
 * */
if(!function_exists('prpimp_generate_tasks_queue')){
    add_action('wp_ajax_prpimp_generate_tasks_queue', 'prpimp_generate_tasks_queue');
    function  prpimp_generate_tasks_queue() {
        if (!isset($_POST['nonce']) || !check_ajax_referer('proradio_pimp-ajax-nonce', 'nonce', false)) {
            wp_send_json_error(array('message' => 'Nonce verification failed.'));
            wp_die();
        }
        if (!isset($_POST['task_id'])){
            wp_send_json_error(array('message' => 'Task missing'));
            wp_die();
        }
        if (get_option('prpimp_import_stopped', false)) {
            wp_send_json_error(array('message' => 'Cannot proceed, import stopped'));
            wp_die();
            return;
        }
        $result = prpimp_import_podcast_feed_to_wp( wp_kses( wp_unslash($_POST['task_id']), array() ) );
        wp_send_json($result);
        wp_die();
    }
}

/***
 * =============================================
 * Extract tasks and feed URLs
 * =============================================
 * */
if(!function_exists('prpimp_get_tasks_list')){
    add_action('wp_ajax_prpimp_get_tasks_list', 'prpimp_get_tasks_list');
    



    function prpimp_get_tasks_list() {
        if (!isset($_POST['nonce']) || !check_ajax_referer('proradio_pimp-ajax-nonce', 'nonce', false)) {
            wp_send_json_error(array('message' => 'Nonce verification failed.'));
            return;
        }
        /**
         * ================================================
         * Usage
         * ================================================
         * */
        $max_uses = 3;
        $today    = date('Y-m-d');
        $posts_per_page = 200;
        $service  = function_exists('prpimp_whmcs_check_service_standalone') ? prpimp_whmcs_check_service_standalone(base64_decode(PRORADIO_PIMP_SERVICE_NAME))  : false;
        if ($service && strtotime($service) > time()) {
            $max_uses = 9999;
        } else {
            global $wpdb;
            $wpdb->query($wpdb->prepare(
                "DELETE FROM {$wpdb->options} WHERE option_name LIKE %s AND option_name < %s",
                'prpimp_usage_%',
                'prpimp_usage_' . $today
            ));
             $posts_per_page = 3;
            $daily_uses = get_option("prpimp_usage_$today", 0);
            update_option("prpimp_usage_$today", $daily_uses + 1);
        }
        if ($daily_uses >= $max_uses) {
            wp_send_json_error(array('message' => 'Max daily use reached.'));
        }




        /**
         * ================================================
         * FETCH THE TASKS
         * ================================================
         * */
        update_option('prpimp_import_stopped', false);
        $args  = array(
            'posts_per_page' => $posts_per_page,
            'post_type'   => 'prpimp-task',
            'post_status' => array('publish'), // Include drafts & trashed
            'meta_query'  => array(
                array(
                    'key'     => 'prpimp-auto',
                    'value'   => '1',
                    'compare' => '=',
                ),
            ),
            'fields'      => 'ids',
        );
        $query = new WP_Query($args);
        $task  = [];

        if ($query->have_posts()) {
            foreach ($query->posts as $post_id) {
                $task[$post_id] = [
                    'prpimp-feed-url' => get_post_meta($post_id, 'prpimp-feed-url', true),
                ];
            }
            wp_send_json($task);
        } else {
            wp_send_json_error(array('message' => 'No tasks available.'));
        }

        wp_die();
    }
}



/***
 * =============================================
 * Stop import trigger
 * =============================================
 * */
add_action('wp_ajax_prpimp_stop_import', 'prpimp_stop_import');
function prpimp_stop_import() {
     if (!isset($_POST['nonce']) ){
        wp_send_json_error(array('message' => 'Nonce missing'));
        wp_die();
     }
    if (!isset($_POST['nonce']) || !check_ajax_referer('proradio_pimp-ajax-nonce', 'nonce', false)) {
        wp_send_json_error(array('message' => 'Nonce verification failed.'));
        wp_die();
    }
    update_option('prpimp_import_stopped', true);
    if( get_option('prpimp_import_stopped', false) ){

       wp_send_json('Import stopped successfully');
    } else {
        wp_send_json_error(array('message' => 'Improt not stopped'));
    }
    wp_die();
}



























































