<?php 
/**
 * @package ProRadio Podacst Importer
 * */

if ( ! defined( 'ABSPATH' ) ) {die( '-1' );}

/**
 *  =============================================
 * 	Import function
 * 	=============================================
 */


if(!function_exists('prpimp_import_podcast_feed_to_wp')){
function prpimp_import_podcast_feed_to_wp($post_id = null) {

	if( !$post_id ){
		update_post_meta( $post_id, 'prpimp-log', 'post_id missing' );
		return false;
	}

	if ( get_option('prpimp_import_stopped', false) ) {
        update_post_meta( $post_id, 'prpimp-log', 'Aborted' );
        wp_send_json(array('message' => 'prpimp_import_stopped: '.get_option('prpimp_import_stopped', false)));
        wp_die();
		return false;
    } 

	

	// Global option
	$import_posttype	= get_option('proradio_pimp_posttype', 'podcast');
	$limit 				= get_option('proradio_pimp_batchquantity',3); if( !$limit 	){ $limit = 1; }


	// Single feed options
	update_post_meta( $post_id, 'prpimp-start', current_time('Y-m-d H:i:s'));
	$log 				= ['date' => current_time('Y-m-d')];
	$auto 				= get_post_meta( $post_id, 'prpimp-auto', true); if(!$auto 	){ return false; }
	$feed_url 			= get_post_meta( $post_id, 'prpimp-feed-url', true );
	$import_artwork 	= get_post_meta( $post_id, 'prpimp-artwork-import', true);
	$striplinks 		= get_post_meta( $post_id, 'prpimp-striplinks', true);
	$artworkr_default 	= get_post_meta( $post_id, 'prpimp-artwork-default', true);

	/**
	 * =======================================================
	 * Download XML of the feed
	 * =======================================================
	 * */
	$response = wp_remote_get($feed_url);
	$log['feed'] = $feed_url;
	if (is_wp_error($response)) {
		$log['error'] = $response;
		update_post_meta( $post_id, 'prpimp-log', $response );
		return;
	}
	$xml_content = wp_remote_retrieve_body($response);
	if(!function_exists('simplexml_load_string')){
		$log['simplexml_error'] = 'Missing simplexml';
		update_post_meta( $post_id, 'prpimp-log', 'Missing library simplexml' );
		return false;
	}
	$xml = simplexml_load_string($xml_content);
	if ($xml === false) {
		return;
	}

	$limit = intval( $limit  );
	if( $limit < 1){
		$limit = 1;
	}
	if( $limit > 50){
		$limit = 50;
	}
	
	$global_image = false;
	if( $xml->channel->image ){
		if( $xml->channel->image->url ){
			$global_image =  html_entity_decode(wp_specialchars_decode((string)  $xml->channel->image->url, ENT_QUOTES));
		}
	}

	$global_channek_link = false;
	if( $xml->channel->link ){
		$global_channek_link =  html_entity_decode(wp_specialchars_decode((string)  $xml->channel->link, ENT_QUOTES));
	}



	$namespace = $xml->getNamespaces(true);
	$channel_itunes = $xml->channel->children($namespace['itunes']);


	 /**
	 * =======================================================
	 * Generate missing category if enabled
	 * =======================================================
	 * */
	$import_taxonomy	= get_option('proradio_pimp_taxonomy', 'podcastfilter');
	
	$category_term = array();
	if ( $import_taxonomy ){


		/**
		 * =======================================================
		 * Categories from podcast feed
		 * =======================================================
		 * */
		if( get_post_meta($post_id,'prpimp-importcategory',true) ){
			if(  $channel_itunes->category ){
				$category_name = (string) $channel_itunes->category->attributes()->text[0];
			}
			if ($category_name && empty($category_name)) {
			    $category_name = false;
			}
			$log['category_name'] = $category_name;
			if (!empty($category_name) && taxonomy_exists($import_taxonomy)) {
				$term = get_term_by('name', $category_name, $import_taxonomy);
				if (!$term) {
					$new_term = wp_insert_term($category_name, $import_taxonomy);
					if (!is_wp_error($new_term)) {
						$category_term[] = $new_term['term_id'];
					}
				} else {
					$category_term[] = $term->term_id;
				}
			}
		}

		 /**
		 * =======================================================
		 * Manually selected terms taxonomy
		 * =======================================================
		 * */
		$terms = wp_get_post_terms($post_id, $import_taxonomy);
		if (!is_wp_error($terms)) {
		    foreach ($terms as $term) {
		    	$log['manual_category_link'] = $term->term_id;
		        $category_term[] = $term->term_id; // preselected category
		    }
		}
	}



	


	
	 /**
	 * =======================================================
	 * Parse all items
	 * =======================================================
	 * */
	$count =0;
	$skipped = 0;
	foreach ($xml->channel->item as $item) {
		wp_cache_flush(); 
		if( $count >= $limit ){
			break;
		}

		if ( get_option('prpimp_import_stopped', false) ) {
	        $log_item['error'] =  'Stopped';
			$skipped += 1;
			$log[] = $log_item;
			continue;
	    } 
		
		$namespace = $item->getNameSpaces(true);
		$title = html_entity_decode(wp_specialchars_decode((string) $item->title, ENT_QUOTES));
		
		$pub_date = (string) $item->pubDate;
		$link = (string) $item->enclosure['url'];
		if(!$link ){
			$log_item['error'] =  'Missing enclosure URL. Skipping';
			$skipped += 1;
			$log[] = $log_item;
			continue;
		}
		$itunes = $item->children($namespace['itunes']);
		$author = (string) $item->children('itunes', true)->author;
		if (empty($author)) {
			$author = false;
		}
		
		/**
		 * =======================================================
		 * Check duplicates
		 * =======================================================
		 * */
		$args = array(
			'title' => esc_html($title),
		    'post_type'   => $import_posttype,
		     'post_status' => array('publish', 'pending', 'draft', 'future', 'private', 'inherit'),
		    'fields'      => 'ids',
		    'suppress_filters' => true,
		);
		$query = new WP_Query($args);
		
		/**
		 * =======================================================
		 * Search for duplicates
		 * =======================================================
		 * */
		if (!empty($query->posts)) {
			$skipped += 1;
		} else {
			$log_item = [];

			/**
			 * =======================================================
			 * Insert post
			 * =======================================================
			 * */
			$description = '';
			if( $item->description && false != get_post_meta( $post_id, 'prpimp-importcontent', true ) ){
				$description = (string) $item->description;	
				$description = preg_replace('#<style.*?>.*?</style>#is', '', $description);
    			$description = preg_replace('#<script.*?>.*?</script>#is', '', $description);
			}
			if( $striplinks && $description ){
				$description = preg_replace('#<a.*?>(.*?)</a>#is', '$1', $description);
			}
			

			if(false != get_post_meta( $post_id, 'prpimp-linksource', true )){
				$linksource = "\n".esc_html__('Source','proradio-podcast-importer').': <a href="'.esc_url($global_channek_link).'" rel="external nofollow" target="_blank">'.esc_attr($global_channek_link).'</a>';
				$description .= $linksource;
			}



			$post_data = array(
				'post_title'    => esc_html($title),
				'post_content'  => wp_kses_post($description),
				'post_status'   => 'publish',
				'post_date'     => gmdate('Y-m-d H:i:s', strtotime($pub_date)),
				'post_type'     => $import_posttype,
				'meta_input'    => array(
					'enclosure' => esc_url($link),
					'_podcast_date' => esc_attr($pub_date),
					'_podcast_resourceurl' => esc_url($link),
					'_podcast_artist' =>  esc_attr($author),
					'_podcast_artwork' =>  esc_url($image),
					'buyurl' => esc_url ( $global_channek_link ),
					'proradio-pimp-imported' => 1,
				),
			);
			$insert_post_id = wp_insert_post($post_data);

			// Assign taxonomy term to post
			if ($insert_post_id && $category_term &&  $import_taxonomy) {
				$log_item['set_tax_term'] =  $category_term;
				wp_set_post_terms($insert_post_id, $category_term, $import_taxonomy);
			}

			/**
			 * =======================================================
			 * Set featured image
			 * =======================================================
			 * */
			if($import_artwork && $insert_post_id  && !is_wp_error($insert_post_id )){
				$log_item['start_import_artwork'] = '1';
				$image = (string) $itunes->image->attributes()->href[0];
				if (empty($image)) {
					$image = false;
				}
				if(!$image && $global_image){
					$log_item['global_image_url'] = $global_image;
					$image = $global_image;
				}
				if($image){
					$featured = prpimp_media_sideload_image( $image, $insert_post_id, '[Podcast artwork '.$insert_post_id.'] '.$title, 'id' );
					$log_item['import_artwork_featured'] = $featured;
					if(!is_wp_error($featured)){
						$added_meta = set_post_thumbnail($insert_post_id, $featured);
						if( !$added_meta ){
							$log_item['import_artwork_featured_error'] = $featured;
						} else {
							// $log_item['import_artwork_featured_success'] = $added_meta;
						}
					} else {
						$log_item['thumbnail'] = 'error';
						// print_r( $featured );
					}
				}else {
					$log_item['thumbnail'] = 'missing';
				}
			}

			// If have custom artwork and import_artwork was failed or disabled
			if( !$import_artwork || $log_item['thumbnail'] == 'error' || $log_item['thumbnail'] == 'missing' ){
				if( $artworkr_default && !empty($artworkr_default)){
					set_post_thumbnail($insert_post_id, $artworkr_default);
				}
			}
			if( $insert_post_id  ){
				$count +=1;
			}
			$log[] = $log_item;
			
		} // not duplicated
		
	}
	$total_items = 0;
	if (  $xml->channel->item ){
		$total_items = count( $xml->channel->item ) ;
	}

	/**
	 * =========================================
	 * Evaluate the results
	 * =========================================
	 * */
	$count = intval($count);
	$skipped = intval($skipped);
	$log['limit'] = $limit;
	$log['total_imported'] = $count;
	$log['skipped'] = $skipped;
	$log['total_items'] = $total_items;
	if ($count == $limit 
	    || ($count + $skipped) >= $total_items // Processed all available items
	    || $total_items < $limit // Fewer items available than the limit
	) {
	   	$result = 'success';
	} else {
	    $result = 'fail';
	}
	$log['result'] = $result;

	/**
	 * =========================================
	 * Output debug
	 * =========================================
	 * */
	$log_text = json_encode($log);
	update_post_meta( $post_id, 'prpimp-log', $log_text );
	if( $result === 'success' ){
		update_post_meta( $post_id, 'prpimp-last', current_time('Y-m-d H:i:s'));
	}

	return $log;
}}