<?php 
/**
 * @package Pro.Radio Podcast Importer
 * Run all import tasks
 * 
 * */
if ( ! defined( 'ABSPATH' ) ) { exit; }

// Schedule an event if it's not already scheduled
// function prpimp_schedule_cron() {
//     if (!wp_next_scheduled('prpimp_run_import')) {
//         wp_schedule_event(time(), 'twicedaily', 'prpimp_run_import');
//     }
// }
// add_action('wp', 'prpimp_schedule_cron');


/**
 * ================================================
 * Hook the function to the cron event
 * ================================================
 * */
add_action('prpimp_run_import', 'prpimp_run_import_callback');
function prpimp_run_import_callback() {
    update_option('prpimp_import_stopped', false);
    $start_time = current_time('Y-m-d H:i:s');
    $args = array(
        'post_type'   => 'prpimp-task',
        'post_status' => 'publish',
        'meta_query'  => array(
            array(
                'key'     => 'prpimp-auto',
                'value'   => '1',
                'compare' => '=',
            ),
        ),
        'fields'      => 'ids',
    );
    $query = new WP_Query($args);
    if ($query->have_posts()) {
        foreach ($query->posts as $post_id) {
            $import_success = prpimp_import_podcast_feed_to_wp($post_id);
        }
    }
    $end_time = current_time('Y-m-d H:i:s');
     if ($import_success) {
        prpimp_log_cron_execution("✅ Import successful. Started at: $start_time, Ended at: $end_time.");
    } else {
        prpimp_log_cron_execution("❌ Import failed. Started at: $start_time, Ended at: $end_time.");
    }
}

/**
 * ================================================
 * Log cron execution
 * ================================================
 * */
function prpimp_log_cron_execution($message) {
    $log_title = 'Cron Run - ' . gmdate('Y-m-d H:i:s');
    $log_id = wp_insert_post(array(
        'post_type'    => 'prpimp-cron-log',
        'post_title'   => $log_title,
        'post_content' => $message,
        'post_status'  => 'publish',
    ));
    return $log_id;
}

/**
 * ================================================
 * Handle form submissions sto start and stop croon
 * ================================================
 * */

function prpimp_add_cron_intervals($schedules) {
    $cron_frequency = intval( get_option( 'proradio_pimp_cronfreq', 60 ) ); // Default to 60 minutes if missing
    if ($cron_frequency < 15) {
        $cron_frequency = 15; // Ensure minimum 15 minutes
    }
    $interval_seconds = $cron_frequency * 60; // Convert to seconds
    /* translators: %d: number of minutes */
    $schedules['prpimp_scheduled_time'] = array(
        'interval' => $interval_seconds,
        /* translators: %d: minutes */
        'display'  => sprintf( __('Every %d minutes', 'proradio-podcast-importer'), $cron_frequency ),
    );
    
    return $schedules;
}


add_filter('cron_schedules', 'prpimp_add_cron_intervals');


function prpimp_handle_cron_settings() {
    if (!isset($_POST['prpimp_cron_nonce']) ) {
        return;
    }
     if (!check_admin_referer('prpimp_cron_nonce_action', 'prpimp_cron_nonce')) {
        return;
     }
    if (isset($_POST['prpimp_enable_cron'])) {
        if (!wp_next_scheduled('prpimp_run_import')) {
            wp_schedule_event(time(), 'prpimp_scheduled_time', 'prpimp_run_import');
        }
        add_action('admin_notices', function() {
             ?><div class="notice notice-success is-dismissible"><p><?php esc_html_e('Cron job activated successfully.', 'proradio-podcast-importer'); ?></p></div><?php  
        });
    }
    if (isset($_POST['prpimp_disable_cron'])) {
        wp_clear_scheduled_hook('prpimp_run_import');
        add_action('admin_notices', function() {
            ?><div class="notice notice-success is-dismissible"><p><?php esc_html_e('Cron job removed successfully.', 'proradio-podcast-importer'); ?></p></div><?php  
        });
    }
}
add_action('admin_init', 'prpimp_handle_cron_settings');


/**
 * ================================================
 * Cron log
 * ================================================
 * */
function prpimp_register_cron_log_cpt() {
    register_post_type('prpimp-cron-log', array(
        'labels'      => array(
            'name'          => __('Cron Logs', 'proradio-podcast-importer'),
            'singular_name' => __('Cron Log', 'proradio-podcast-importer'),
        ),
        'public'      => false, // Hidden from front-end
        'show_ui'     => false, // Hidden from admin UI
        'supports'    => array('title', 'editor'), // Store log messages
    ));
}
add_action('init', 'prpimp_register_cron_log_cpt');


/**
 * ================================================
 * Display last log
 * ================================================
 * */
function prpimp_display_cron_logs() {
    $logs = get_posts(array(
        'post_type'   => 'prpimp-cron-log',
        'posts_per_page' => 1, 
        'order'       => 'DESC',
    ));
    if (!empty($logs)) {
        echo '<ul>';
        foreach ($logs as $log) {
            echo '<li><strong>' . esc_html($log->post_title) . '</strong><br>' . esc_html($log->post_content) . '</li><hr>';
        }
        echo '</ul>';
    } else {
        echo '<p>'.esc_html__('No logs found','proradio-podcast-importer').'</p>';
    }
    echo '</div>';
}

