<?php 
/**
 * @package Pro.Radio Podcast Importer
 * 
 * Ajax function:
 * store a song in the Songs CPT
 * 
 * */
if ( ! defined( 'ABSPATH' ) ) { exit; }

/***
 * =============================================
 * Ajax function to store song from charts
 * =============================================
 * */

function prpimp_media_sideload_image( $file, $post_id = 0, $desc = null, $return_type = 'html' ) {
    if (get_option('prpimp_import_stopped', false)) {
        return false;
    }
    if ( ! empty( $file ) ) {
        // Ensure URL is without query parameters (optional, depending on use case)
        $file = strtok( $file, '?' );  // Removes any query parameters (e.g., ?aid=rss_feed)

        // Allowed image extensions
        $allowed_extensions = array( 'jpg', 'jpeg', 'jpe', 'png', 'gif', 'webp' );
        $allowed_extensions = apply_filters( 'image_sideload_extensions', $allowed_extensions, $file );
        $allowed_extensions = array_map( 'preg_quote', $allowed_extensions );
        
        // Validate image URL extension
        preg_match( '/[^\?]+\.(' . implode( '|', $allowed_extensions ) . ')\b/i', $file, $matches );
        if ( ! $matches ) {
            return new WP_Error( 'image_sideload_failed', __( 'Invalid image URL.', 'proradio-podcast-importer' ) );
        }

        // Prepare to download and upload the image
        $file_array = array();
        $file_array['name'] = wp_basename( $matches[0] );
        $file_array['tmp_name'] = download_url( $file );
        
        // Handle potential error while downloading
        if ( is_wp_error( $file_array['tmp_name'] ) ) {
            $error_message = $file_array['tmp_name']->get_error_message();
            // Log the error for debugging
            // error_log( 'Error downloading image: ' . $error_message );
            return new WP_Error( 'image_sideload_failed', __( 'Failed to download the image: ' , 'proradio-podcast-importer') . $error_message );
        }

        // Try uploading the image to the media library
        $id = media_handle_sideload( $file_array, $post_id, $desc );
        
        // Handle error in case upload fails
        if ( is_wp_error( $id ) ) {
            wp_delete_file( $file_array['tmp_name'] );  // Clean up temporary file
            $error_message = $id->get_error_message();
            // Log the error for debugging
            // error_log( 'Error uploading image: ' . $error_message );
            return new WP_Error( 'image_sideload_failed', __( 'Failed to upload the image: ', 'proradio-podcast-importer' ) . $error_message );
        }

        // Add the source URL as post meta
        add_post_meta( $id, '_source_url', $file );

        // Return the image ID or HTML as needed
        if ( 'id' === $return_type ) {
            return $id;
        }

        return $file_array['tmp_name']; // Return the temporary file path if needed
    }
}
